
local addon_name, ns = ...

local Printer = LibStub:GetLibrary("OakPrint-1.0")(addon_name .. " Rotation")
local rotation = {}

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Rotation managment
--------------------------------------------------------------------------------

local function IndexInQueue(queue, val)
    for i, v in ipairs(queue.frames) do
        if v.name == val then
            return i
        end
    end
end

--[[
local function modlen(length, n)
    return ((n - 1) % length) + 1
end
]]

local function NextIndexIncremental(queue, next_index, index, data)
-- when inter-client comms are up, test HasFullControll() for each interrupter
    local off_cd_first = next_index
    local func = function(i)
        local name = queue.frames[i].name
        local t = data[name]
        if UnitIsDead(name) then
            return
        end
        if t < GetTime() then
            return true, i
        elseif t < off_cd_first then
            off_cd_first = i
        end
    end
    for i=next_index, #queue.frames do
        local r, ind = func(i)
        if r then return ind end
    end
    for i = 1, next_index do
        local r, ind = func(i)
        if r then return ind end
    end
    return off_cd_first
end

--[[
local function NextIndexDumb(queue, next_index, index, data)
    if index and next_index == index then
        local n = modlen(#queue.frames, index + 1)
        return n
    end
    return next_index
end
]]

local function Announce(disp, fmt, ...)
    assert(disp, "Usage: Announce(display, [fmt, ...])")
    Printer.debug(2, "Announcing %s...", disp.name)
    local msg
    if fmt then
        msg = format(fmt, ...)
    else
        local name = disp.queue.frames[disp.rotation.next_index].name
        msg = format("Next interrupt is %s", name)
    end
    local t = (UnitIsGroupLeader('player') or UnitIsGroupAssistant('player'))
             and "RAID_WARNING" or "YELL"
    Printer.debug(2, "Announcing '%s' to %s", msg, t)
    -- Do we need pcall? Can't remember why...
    pcall(SendChatMessage, msg, t)
end

function rotation.AdvanceRotation(disp, name, data)
    if not disp.rotation then return end
    Printer.debug(2, "Advancing rotation for %s", disp.name)
    local cur_index = disp.rotation.next_index
    local queue = disp.queue
    local index = IndexInQueue(queue, name)
    local next_index = NextIndexIncremental(queue, cur_index, index, data)
    Printer.debug(3, "%s - cur_index: %d, next_index: %d", disp.name, cur_index, next_index)
    if not (next_index == cur_index) then
        disp.queue.frames[cur_index].bg:SetTexture(unpack(ns.settings.colour.default))
        disp.queue.frames[next_index].bg:SetTexture(unpack(ns.settings.colour.highlight))
    end
    disp.rotation.next_index = next_index
    if disp.rotation.announce then
        Announce(disp)
    end
end

function rotation.AnnounceOrder(disp)
    Printer.debug(2, "Sharing order for %s", disp.name)
    local order = {}
    local fmt
    for k, v in ipairs(disp.queue.frames) do
        tinsert(order, v.name)
        fmt = ((fmt and fmt .. " -- ") or "-- ") .. k .. " %s"
    end
    fmt = fmt .. " --"
    Announce(disp, fmt, unpack(order))
end


ns.rotation = rotation
ns.rotation.Printer = Printer

-- TODO: 
--       ideas for NextIndex:
--              a min cd remaining time?
